//
//  marquee.c - a simple marquee with four different messages.
//
//  Jeffrey Carl Faden 2011
//  http://jeffreyatw.com
//
//  This work is licensed under a Creative Commons
//  Attribution-NonCommercial-ShareAlike 3.0 Unported License.
//  http://creativecommons.org/licenses/by-nc-sa/3.0/
//
//  Hardware requirements: Mignonette v2.0
//
//  Revision History
//
//  20110827 - Initial release
//

#include <avr/io.h>			/* this takes care of definitions for our specific AVR */
#include <avr/pgmspace.h>	/* needed for printf_P, etc */
#include <avr/interrupt.h>	/* for interrupts, ISR macro, etc. */
#include <string.h>			// for memset
#include <ctype.h>          // for toupper

#include "mydefs.h"
#include "iodefs.h"

#include "miggl.h"		/* Mignonette Game Library */

// for _delay_us() macro  (note: this gets F_CPU define from miggl.h)
#include <util/delay.h>

#include "messages.h"

char *message = messageA;

// One row of the currently selected character.
unsigned char currentchar;

// The pixel/bit width of the current selected character.
uint8_t currentcharwidth;

// The message array that will hold the entire buffer.
unsigned char messagearray[5][100];

// Used to calculate current and final message array length, in bits.
unsigned long int current_messagearray_length;

uint8_t colors[3] = {RED, GREEN, YELLOW};

void setcurrentchar(unsigned char c, uint8_t row);
void draw_bit_line (unsigned long int i, uint8_t line, uint8_t color);
void draw_bitmap (unsigned long int i, uint8_t color);
void generatemessagearray();

// Sets one line of the currently selected char.
void setcurrentchar(unsigned char c, uint8_t row) {
    switch (c) {
        case '$':
            switch (row) {
                case 0:
                    currentchar = 0x20;
                    break;
                case 1:
                    currentchar = 0x70;
                    break;
                case 2:
                    currentchar = 0xC0;
                    break;
                case 3:
                    currentchar = 0x30;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '*':
            switch (row) {
                case 0:
                    currentchar = 0xA0;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0xA0;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '=':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0xE0;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0xE0;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '%':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0x10;
                    break;
                case 2:
                    currentchar = 0x20;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x48;
                    break;
            }
            currentcharwidth = 5;
            break;
        case '"':
            switch (row) {
                case 0:
                    currentchar = 0xA0;
                    break;
                case 1:
                    currentchar = 0xA0;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '\'':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 1;
            break;
        case '#':
            switch (row) {
                case 0:
                    currentchar = 0x50;
                    break;
                case 1:
                    currentchar = 0xF8;
                    break;
                case 2:
                    currentchar = 0x50;
                    break;
                case 3:
                    currentchar = 0xF8;
                    break;
                case 4:
                    currentchar = 0x50;
                    break;
            }
            currentcharwidth = 5;
            break;
        case '@':
            switch (row) {
                case 0:
                    currentchar = 0x70;
                    break;
                case 1:
                    currentchar = 0x88;
                    break;
                case 2:
                    currentchar = 0xB8;
                    break;
                case 3:
                    currentchar = 0xA8;
                    break;
                case 4:
                    currentchar = 0x70;
                    break;
            }
            currentcharwidth = 5;
            break;
        case '&':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x68;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x68;
                    break;
            }
            currentcharwidth = 5;
            break;
        case ';':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 1;
            break;
        case ' ':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x00;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 2;
            break;
        case '?':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x10;
                    break;
                case 2:
                    currentchar = 0x60;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x40;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '!':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 1;
            break;
        case '.':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x00;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 1;
            break;
        case ',':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x00;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 2;
            break;
        case '|':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 1;
            break;
        case '{':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '}':
            switch (row) {
                case 0:
                    currentchar = 0xC0;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x20;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0xC0;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '<':
            switch (row) {
                case 0:
                    currentchar = 0x20;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x20;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '>':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x20;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '[':
            switch (row) {
                case 0:
                    currentchar = 0xC0;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0xC0;
                    break;
            }
            currentcharwidth = 2;
            break;
        case ']':
            switch (row) {
                case 0:
                    currentchar = 0xC0;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x40;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0xC0;
                    break;
            }
            currentcharwidth = 2;
            break;
        case '`':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 2;
            break;
        case '^':
            switch (row) {
                case 0:
                    currentchar = 0x40;
                    break;
                case 1:
                    currentchar = 0xA0;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '~':
            switch (row) {
                case 0:
                    currentchar = 0x50;
                    break;
                case 1:
                    currentchar = 0xA0;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '-':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x00;
                    break;
                case 2:
                    currentchar = 0xE0;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '/':
            switch (row) {
                case 0:
                    currentchar = 0x08;
                    break;
                case 1:
                    currentchar = 0x10;
                    break;
                case 2:
                    currentchar = 0x20;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 5;
            break;
        case '\\':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x20;
                    break;
                case 3:
                    currentchar = 0x10;
                    break;
                case 4:
                    currentchar = 0x08;
                    break;
            }
            currentcharwidth = 5;
            break;
        case ':':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 1;
            break;
        case '+':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0xE0;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x00;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '_':
            switch (row) {
                case 0:
                    currentchar = 0x00;
                    break;
                case 1:
                    currentchar = 0x00;
                    break;
                case 2:
                    currentchar = 0x00;
                    break;
                case 3:
                    currentchar = 0x00;
                    break;
                case 4:
                    currentchar = 0xF0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '(':
            switch (row) {
                case 0:
                    currentchar = 0x40;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0x40;
                    break;
            }
            currentcharwidth = 2;
            break;
        case ')':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x40;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 2;
            break;
        case 'A':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0xF0;
                    break;
                case 4:
                    currentchar = 0x90;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'B':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0xE0;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'C':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 3;
            break;
        case 'D':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'E':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0xE0;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 3;
            break;
        case 'F':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0xE0;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 3;
            break;
        case 'G':
            switch (row) {
                case 0:
                    currentchar = 0x70;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0xB0;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x70;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'H':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0xF0;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x90;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'I':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x40;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 3;
            break;
        case 'J':
            switch (row) {
                case 0:
                    currentchar = 0x30;
                    break;
                case 1:
                    currentchar = 0x10;
                    break;
                case 2:
                    currentchar = 0x10;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'K':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0xA0;
                    break;
                case 2:
                    currentchar = 0xC0;
                    break;
                case 3:
                    currentchar = 0xA0;
                    break;
                case 4:
                    currentchar = 0x90;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'L':
            switch (row) {
                case 0:
                    currentchar = 0x80;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x80;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 3;
            break;
        case 'M':
            switch (row) {
                case 0:
                    currentchar = 0x88;
                    break;
                case 1:
                    currentchar = 0xD8;
                    break;
                case 2:
                    currentchar = 0xA8;
                    break;
                case 3:
                    currentchar = 0x88;
                    break;
                case 4:
                    currentchar = 0x88;
                    break;
            }
            currentcharwidth = 5;
            break;
        case 'N':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0xD0;
                    break;
                case 2:
                    currentchar = 0xB0;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x90;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'O':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'P':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0xE0;
                    break;
                case 4:
                    currentchar = 0x80;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'Q':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0xA0;
                    break;
                case 4:
                    currentchar = 0x50;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'R':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0xE0;
                    break;
                case 4:
                    currentchar = 0x90;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'S':
            switch (row) {
                case 0:
                    currentchar = 0x70;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0x60;
                    break;
                case 3:
                    currentchar = 0x10;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'T':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x40;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x40;
                    break;
            }
            currentcharwidth = 3;
            break;
        case 'U':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'V':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0xA0;
                    break;
                case 3:
                    currentchar = 0xA0;
                    break;
                case 4:
                    currentchar = 0x40;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'W':
            switch (row) {
                case 0:
                    currentchar = 0x88;
                    break;
                case 1:
                    currentchar = 0xA8;
                    break;
                case 2:
                    currentchar = 0xA8;
                    break;
                case 3:
                    currentchar = 0xA8;
                    break;
                case 4:
                    currentchar = 0x50;
                    break;
            }
            currentcharwidth = 5;
            break;
        case 'X':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x60;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x90;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'Y':
            switch (row) {
                case 0:
                    currentchar = 0x90;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x70;
                    break;
                case 3:
                    currentchar = 0x10;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        case 'Z':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x20;
                    break;
                case 2:
                    currentchar = 0x40;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 3;
            break;
        case '0':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x90;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '1':
            switch (row) {
                case 0:
                    currentchar = 0xC0;
                    break;
                case 1:
                    currentchar = 0x40;
                    break;
                case 2:
                    currentchar = 0x40;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x40;
                    break;
            }
            currentcharwidth = 2;
            break;
        case '2':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x10;
                    break;
                case 2:
                    currentchar = 0x60;
                    break;
                case 3:
                    currentchar = 0x80;
                    break;
                case 4:
                    currentchar = 0xF0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '3':
            switch (row) {
                case 0:
                    currentchar = 0xE0;
                    break;
                case 1:
                    currentchar = 0x10;
                    break;
                case 2:
                    currentchar = 0x60;
                    break;
                case 3:
                    currentchar = 0x10;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '4':
            switch (row) {
                case 0:
                    currentchar = 0x20;
                    break;
                case 1:
                    currentchar = 0x60;
                    break;
                case 2:
                    currentchar = 0xA0;
                    break;
                case 3:
                    currentchar = 0xF0;
                    break;
                case 4:
                    currentchar = 0x20;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '5':
            switch (row) {
                case 0:
                    currentchar = 0xF0;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0xE0;
                    break;
                case 3:
                    currentchar = 0x10;
                    break;
                case 4:
                    currentchar = 0xE0;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '6':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x80;
                    break;
                case 2:
                    currentchar = 0xE0;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '7':
            switch (row) {
                case 0:
                    currentchar = 0xF0;
                    break;
                case 1:
                    currentchar = 0x10;
                    break;
                case 2:
                    currentchar = 0x20;
                    break;
                case 3:
                    currentchar = 0x40;
                    break;
                case 4:
                    currentchar = 0x40;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '8':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x60;
                    break;
                case 3:
                    currentchar = 0x90;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        case '9':
            switch (row) {
                case 0:
                    currentchar = 0x60;
                    break;
                case 1:
                    currentchar = 0x90;
                    break;
                case 2:
                    currentchar = 0x70;
                    break;
                case 3:
                    currentchar = 0x10;
                    break;
                case 4:
                    currentchar = 0x60;
                    break;
            }
            currentcharwidth = 4;
            break;
        default:
            currentchar = 0x00;
            currentcharwidth = 0;
            break;
    };
};

//
// crude delay of 1 to 255 ms
//
void delay_ms(uint8_t ms)
{
	ms++;
    
	while (--ms) {
		_delay_ms(1);	// get 1ms delay from library macro (see <util/delay.h>)
	}
}

// Draws one line, offset by current progress through message array buffer.
void draw_bit_line (unsigned long int i, uint8_t line, uint8_t color) {
	char x;
	for (x = 0; x < XSCREEN; x++) {
        // Find current index by dividing bytes into bits,
        // then shift to ones position
		if ((messagearray[line][(i + x) / 8] >> (7 - (i + x) % 8)) & 1)
            setcolor(color);
        else
            setcolor(BLACK);
        drawpoint(x, line);
	}
}

//
// draws a whole bitmap into the screen with a given color
//
void draw_bitmap (unsigned long int i, uint8_t color) {
	char y;
	for (y = 0; y < YSCREEN; y++)
		draw_bit_line(i, y, color);
}

// Concatenate every character into a single 2d array of bits.
void generatemessagearray() {
    uint8_t i;
    // j must be able to accomodate message length
    unsigned long int j, oldcurrent_messagearray_length;
    unsigned char currentbits;
    
    memset(messagearray, 0, sizeof(messagearray[0][0]) * sizeof(messagearray));
    
    for (i = 0; message[i]; i++) {
        message[i] = toupper(message[i]);
    }
    
    // Iterate through every row
    for (i = 0; i < YSCREEN; i++) {
        // Append seven 0's for initial black screen
        current_messagearray_length = XSCREEN;
        // Iterate through every letter of the message
        for (j = 0; message[j]; j++) {    
            setcurrentchar(message[j], i); 
            if (currentcharwidth != 0) {
                
                // Shift current character's bits to the right as many times as
                // the message array's last value is long
                currentbits = currentchar >> (current_messagearray_length % 8);
                
                // Add the currentbits onto the message array's last value
                messagearray[i][current_messagearray_length / 8] += currentbits;
                
                // Store the current message array length
                oldcurrent_messagearray_length = current_messagearray_length;
                
                // Increase the current message array length
                current_messagearray_length += currentcharwidth;
                
                // If there weren't enough spaces to fill the message array's
                // last value...
                if (8 - oldcurrent_messagearray_length % 8 < currentcharwidth) {
                    // Shift current character's bits to the left 8 times...
                    // then shift back as many times as the previous message
                    // array's last value was long
                    currentbits = currentchar << 
                                  (8 - oldcurrent_messagearray_length % 8);
                    
                    // Add the remaining bits to the message array's new last
                    // value
                    messagearray[i][current_messagearray_length / 8] +=
                    currentbits;
                }
                
                // Add letter-spacing
                current_messagearray_length++;
            }
        }
    }
    
    // Remove letter-spacing from end of last character
    current_messagearray_length--;
}

int main() {
    // Long enough for message array length
    unsigned long int i;
    
    avrinit();
    swapinterval(10);
    cleardisplay();
    start_timer1();
    button_init();
    
    generatemessagearray();
        
    i = 0;
    while (1) {
        handlebuttons();
        
        if (ButtonA) {
			message = messageA;
		} else if (ButtonB) {
			message = messageB;
		} else if (ButtonC) {
			message = messageC;
		} else if (ButtonD) {
			message = messageD;
		}
        
        if (ButtonA || ButtonB || ButtonC || ButtonD) {
            generatemessagearray();
            i = 0;
        }
        
        draw_bitmap(i, colors[i % sizeof(colors)/sizeof(int)]);
        delay_ms(50);
        i++;
        if (i == current_messagearray_length) {
            // We've reached the end, start over
            i = 0;
        }
    }
    
    return 0;
}
